## ---- echo = FALSE--------------------------------------------------------------------------------
options(width = 100)


## ---- setup, eval=TRUE,echo=TRUE----------------------------------------------------------------------------------------------------------
## to make this into a word document:
## library(rmarkdown); render(filename.Rmd", output_format="word_document")
options(stringsAsFactors=FALSE,width=140)
library(knitr) ## contains kable
library(MASS)  ## contains mvrnorm
library(panoply) 


## ----loadSolid, eval=TRUE, echo=TRUE------------------------------------------------------------------------------------------------------
data(gcPanCO)
covGC <- cov(t(gcPanCO))

## load("covSOLID.RData")
## covGC <- covSOLID
meanGC <- rep(0, ncol(covGC))
names(meanGC) <- colnames(covGC)

data(genelistPan)
data(gcinfoPan)
data(dgidbPan)
data(reactome)
data(dgiSets)


## ---- simsettings, echo=TRUE, eval=TRUE---------------------------------------------------------------------------------------------------

## TPR drug/genes
pickDrug <- "OLAPARIB"                    ## Drug considered the "true positive" 
cnGenes <- c("ATR","BLM","BRCA1","CHEK2") ## cancer driver genes for Olaparib

npt <- 9           ## 1 case and 8 matched controls
sdExpress <- 2     ## gene-specific over-expression level in the case (sd of genes)
n.dna.events <- 30 ## in addition to RNA drivers, number of random dna drivers (of total 429)

## ---- makeGenesD, echo=TRUE, eval=TRUE, results='asis'------------------------------------------------------------------------------------
brca1 <- colnames(reactome.adj)[reactome.adj["BRCA1",]>0]
chek2 <- colnames(reactome.adj)[reactome.adj["CHEK2",]>0]
atr <- colnames(reactome.adj)[reactome.adj["ATR",]>0]
blm <- colnames(reactome.adj)[reactome.adj["BLM",]>0]
alltbl <- table(c(brca1, chek2,atr,blm))
pickNetGenes <- names(alltbl)[alltbl>1]
kable(data.frame(Genes.Olaparib=pickNetGenes))

## ---- algsettings-------------------------------------------------------------------------------------------------------------------------
set.tailPct <- 0.2     ## pct of RNA genes in case (upper or lower tails) as "drivers" in DMT
set.tailEnd <- "upper" ## direction of tail as RNA drivers (upper, lower, or both) 
set.eventOnly=TRUE     ## in DMT, only use RNA+DNA driver events, if FALSE, uses all 429
set.nsim=100           ## n-simulations for DMT p-value. We suggest using 1000 or more, but fewer here for run-time.

## ---- rnaonly, echo=TRUE, eval=TRUE-------------------------------------------------------------------------------------------------------
meanGC <- rep(0, ncol(covGC))
names(meanGC) <- colnames(covGC)
meanGCcase <- meanGC
sdDrGenes <- sqrt(diag(covGC[pickNetGenes,pickNetGenes])) * sdExpress
meanGCcase[pickNetGenes] <- 0 + sdDrGenes

set.seed(2000)

gcSim1 <- mvrnorm(1, mu=meanGCcase, Sigma=covGC)
gcSim8 <- mvrnorm(npt-1, mu=meanGC, Sigma=covGC)
gcSim <- t(rbind(gcSim1, gcSim8))
colnames(gcSim) <- paste("Null",1:npt,sep="_")

i <- colnames(gcSim)[1]
patient <- i
ptmatch <- colnames(gcSim)[!(colnames(gcSim) %in% i)]

driversRNA <- panGeneSets(caseids=patient,controlids=ptmatch,gcount=gcSim,
                     tailPct=set.tailPct, tailEnd=set.tailEnd, eventOnly=set.eventOnly)

kable(driversRNA[1:10,1:6], row.names=FALSE, caption="Differentially-Expressed Driver Gene Sets, RNA drivers")
  
drugRNA <- panDrugSets(driversRNA,caseids=patient, controlids=ptmatch,
                 gcount=gcSim, gene.gs=reactome.gs,
                 gene.adj=reactome.adj, drug.gs=dgi.gs, drug.adj=dgi.adj,
                 minPathways=0, nsim = set.nsim, tailEnd = set.tailEnd)                    


kable(drugRNA[1:15,1:10], row.names=FALSE, digits=3, caption="Drug Test Results, RNA-only")
  

## ---- drivers4, echo=TRUE, eval=TRUE------------------------------------------------------------------------------------------------------
set.seed(2000)
colnames(gcSim) <- paste("Null",1:npt,sep="_")

i <- colnames(gcSim)[1]
patient <- i
ptmatch <- colnames(gcSim)[!(colnames(gcSim) %in% i)]
cnSim <- data.frame(CHROM=1,Gene.Symbol=rownames(reactome.adj), 
                 matrix(0, nrow=nrow(reactome.adj), ncol=npt))
 
colnames(cnSim)[3:ncol(cnSim)] <- c(patient, ptmatch); 
cnSim[sample(nrow(reactome.adj),n.dna.events),patient] <- 1
cnSim[cnSim$Gene.Symbol %in% cnGenes,patient] <- 1

drivRNADNA <- panGeneSets(caseids=patient,controlids=ptmatch,
                        gene.adj=reactome.adj, drug.adj=dgi.adj,
                        gcount=gcSim,cna=cnSim,tailPct=set.tailPct,
                        tailEnd=set.tailEnd, eventOnly=set.eventOnly)
kable(drivRNADNA[1:10,1:6], row.names=FALSE, caption="Differentially-Expressed Driver Gene Sets from RNA and DNA drivers")
  
drugRNADNA <- panDrugSets(drivRNADNA,caseids=patient, controlids=ptmatch,
                  gcount=gcSim, gene.gs=reactome.gs, gene.adj=reactome.adj,
                  drug.gs=dgi.gs, drug.adj=dgi.adj,
                  minPathways=0, nsim=set.nsim, tailEnd=set.tailEnd)  

kable(drugRNADNA[1:15,1:10], row.names=FALSE, digits=3, caption="Drug Test Results")
       

## ---- panelgenes, echo=TRUE, eval=TRUE----------------------------------------------------------------------------------------------------
set.seed(1000)
panel.genes <- sample(colnames(reactome.adj), size=1000, replace=FALSE)
panel.adj <- reactome.adj[,panel.genes]
panel.adj <- panel.adj[rowSums(panel.adj)>0,]
dim(panel.adj)
 
drivPanel <- panGeneSets(caseids=patient, controlids=ptmatch,
                   gcount=gcSim, cna=cnSim, tailPct=set.tailPct,
                   gene.adj=panel.adj, drug.adj=dgi.adj,
                   tailEnd=set.tailEnd, eventOnly=FALSE)
kable(drivPanel[1:10,1:6], digits=3, row.names=FALSE, caption="DE Driver Gene Sets from Panel Set, RNA + DNA")

drugPanel <- panDrugSets(drivPanel, caseids=patient, controlids=ptmatch,
                   gcount=gcSim, gene.gs=reactome.gs, gene.adj=panel.adj,
                   drug.gs=dgi.gs, drug.adj=dgi.adj, minPathways=0,
                   nsim=set.nsim, tailEnd=set.tailEnd)  
kable(drugPanel[1:10,1:10], digits=3, row.names=FALSE, caption="Drug Test Results from Gene Panel")                  
 

## ---- showGenesA, eval=TRUE, echo=TRUE, results='asis'------------------------------------------------------------------------------------
pickGenesA <- dgi.gs[[pickDrug]]
pickGenesA <- pickGenesA[pickGenesA %in% colnames(reactome.adj)]
kable(data.frame(Genes.SetA=pickGenesA))

## ---- showGenesB, eval=TRUE, echo=TRUE, results='asis'------------------------------------------------------------------------------------
pickGenesB <- names(which(reactome.adj["BRCA2",]>0))
kable(data.frame(Genes.SetB=pickGenesB))

## ---- showGenesC, eval=TRUE, echo=TRUE, results='asis'------------------------------------------------------------------------------------
pickGenesC <- c("BRCA1","RAD21","FANCG","FANCI","SUMO2","H2AFX","ATM")
kable(data.frame(Genes.SetC=pickGenesC))

DriversC=names(which(rowSums(reactome.adj[,pickGenesC])>2))
kable(data.frame(Drivers.SetC=DriversC), caption="Cancer Driver Genes that target 3 or more genes from set C")

## ---- rsession----------------------------------------------------------------------------------------------------------------------------
sessionInfo()

